@@ -225,16 +225,16 @@ update_color_widget_labels_and_visibility (ApplierSet *set, const gchar *value_s
 
 	switch (orientation) {
 	case ORIENTATION_SOLID: /* solid */ 
-		color1_string = _("C_olor"); 
+		color1_string = _("C_olor:"); 
 		two_colors = FALSE; 
 		break;
 	case ORIENTATION_HORIZ: /* horiz */ 
-		color1_string = _("_Left Color"); 
-		color2_string = _("_Right Color"); 
+		color1_string = _("_Left Color:"); 
+		color2_string = _("_Right Color:"); 
 		break;
 	case ORIENTATION_VERT: /* vert  */
-		color1_string = _("_Top Color");
-		color2_string = _("_Bottom Color"); 
+		color1_string = _("_Top Color:");
+		color2_string = _("_Bottom Color:"); 
 		break;
 	default:
 		break;
@@ -430,7 +430,7 @@ create_dialog (ApplierSet *set)
 
 	g_slist_free (group);
 
-	label = gtk_label_new_with_mnemonic (_("_Picture"));
+	label = gtk_label_new_with_mnemonic (_("Select _picture:"));
 	gtk_frame_set_label_widget (GTK_FRAME (WID ("picture_frame")),
 				    label);
 	gtk_label_set_mnemonic_widget (GTK_LABEL (label),
@@ -500,8 +500,12 @@ drag_data_received_cb (GtkWidget *widget, GdkDragContext *context,
 			GnomeVFSURI *uri = (GnomeVFSURI *) uris->data;
 			GConfEntry *entry;
 			GConfValue *value = gconf_value_new (GCONF_VALUE_STRING);
-			
-			gconf_value_set_string (value, gnome_vfs_uri_get_path (uri));
+			gchar *base;
+
+			base = gnome_vfs_unescape_string (gnome_vfs_uri_get_path (uri),
+							  G_DIR_SEPARATOR_S);
+			gconf_value_set_string (value, base);
+			g_free (base);
 
 			/* Hmm, should we bother with changeset here? */
 			gconf_client_set (client, BG_PREFERENCES_PICTURE_FILENAME, value, NULL);
@@ -550,7 +554,6 @@ main (int argc, char **argv)
 	GladeXML       *dialog;
 	GtkWidget      *dialog_win;
 	ApplierSet     *set;
-	GdkPixbuf      *pixbuf;
 
 	static gboolean get_legacy;
 	static struct poptOption cap_options[] = {
@@ -581,14 +584,12 @@ main (int argc, char **argv)
 		setup_dialog (dialog, NULL, set);
 
 		dialog_win = gtk_dialog_new_with_buttons
-			(_("Background Preferences"), NULL, -1,
+			(_("Background Preferences"), NULL, 0,
 			 GTK_STOCK_HELP, GTK_RESPONSE_HELP,
 			 GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
 			 NULL);
-
-		pixbuf = gdk_pixbuf_new_from_file (GNOMECC_DATA_DIR "/icons/background-capplet.png", NULL);
-		gtk_window_set_icon (GTK_WINDOW(dialog_win), pixbuf);
-		gdk_pixbuf_unref (pixbuf);
+		gtk_dialog_set_default_response (GTK_DIALOG (dialog_win),
+			GTK_RESPONSE_CLOSE);
 
 		g_signal_connect (G_OBJECT (dialog_win),
 				    "response",
@@ -607,6 +608,7 @@ main (int argc, char **argv)
 
 		g_object_weak_ref (G_OBJECT (dialog_win), (GWeakNotify) applier_set_free, set);
 		gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog_win)->vbox), WID ("prefs_widget"), TRUE, TRUE, GNOME_PAD_SMALL);
+		capplet_set_icon (dialog_win, "background-capplet.png");
 		gtk_widget_show (dialog_win);
 
 		gtk_main ();
