@@ -49,9 +49,14 @@ const KeyListEntry metacity_key_list[] =
   { "/apps/metacity/window_keybindings/toggle_maximized",          ALWAYS_VISIBLE,  0 },
   { "/apps/metacity/window_keybindings/toggle_shaded",             ALWAYS_VISIBLE,  0 },
   { "/apps/metacity/window_keybindings/close",                     ALWAYS_VISIBLE,  0 },
+  { "/apps/metacity/window_keybindings/minimize",                  ALWAYS_VISIBLE,  0 },
   { "/apps/metacity/window_keybindings/begin_move",                ALWAYS_VISIBLE,  0 },
   { "/apps/metacity/window_keybindings/begin_resize",              ALWAYS_VISIBLE,  0 },
   { "/apps/metacity/window_keybindings/toggle_on_all_workspaces",  N_WORKSPACES_GT, 1 },
+  /* These next three are out of order compared to the schemas file. */
+  { "/apps/metacity/window_keybindings/raise_or_lower",            ALWAYS_VISIBLE,  0 },
+  { "/apps/metacity/window_keybindings/raise",                     ALWAYS_VISIBLE,  0 },
+  { "/apps/metacity/window_keybindings/lower",                     ALWAYS_VISIBLE,  0 },
   { "/apps/metacity/window_keybindings/move_to_workspace_1",       N_WORKSPACES_GT, 1 },
   { "/apps/metacity/window_keybindings/move_to_workspace_2",       N_WORKSPACES_GT, 1 },
   { "/apps/metacity/window_keybindings/move_to_workspace_3",       N_WORKSPACES_GT, 2 },
@@ -70,7 +75,8 @@ const KeyListEntry metacity_key_list[] =
   { "/apps/metacity/window_keybindings/move_to_workspace_down",    N_WORKSPACES_GT, 1 },
   { "/apps/metacity/global_keybindings/switch_windows",            ALWAYS_VISIBLE,  0 },
   { "/apps/metacity/global_keybindings/switch_panels",             ALWAYS_VISIBLE,  0 },
-  { "/apps/metacity/global_keybindings/focus_previous_window",     ALWAYS_VISIBLE,  0 },
+  { "/apps/metacity/global_keybindings/cycle_windows",             ALWAYS_VISIBLE,  0 },
+  { "/apps/metacity/global_keybindings/cycle_panels",              ALWAYS_VISIBLE,  0 },
   { "/apps/metacity/global_keybindings/show_desktop",              ALWAYS_VISIBLE,  0 },
   { "/apps/metacity/global_keybindings/switch_to_workspace_1",     N_WORKSPACES_GT, 1 },
   { "/apps/metacity/global_keybindings/switch_to_workspace_2",     N_WORKSPACES_GT, 1 },
